#include <cstring>
#include <unistd.h>

#include "lock.hh"
#include "md5.h"

using namespace std;

static const int SALTLENGTH = 32;

static
string
randomSalt()
{
  static string saltChars = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ!@#$%^&*()<>?/'";

  string retString = "";
  for (int c = 0; c < SALTLENGTH; c++) {
    retString += saltChars.substr(rand() % saltChars.length(), 1);
  }
  return retString;
}

static
string
int2string(unsigned int num)
{
  string s;
  while (num > 0) {
    s = string(1, '0' + num % 10) + s;
    num /= 10;
  }
  if (s.length() == 0) {
    s = "0";
  }
  return s;
}

static
unsigned int
string2int(string str)
{
  unsigned int r = 0;
  for (unsigned c = 0; c < str.length(); c++) {
    r = (str[c] - '0') + r * 10;
  }
  return r;
}

static
string
charInt2hex(unsigned char i)
{
  static char* hexChars = "0123456789abcdef";
  return string(1, hexChars[(i >> 8) & 0x0f]) + string(1, hexChars[i & 0x0f]);
}

static
string
digestMd5(string data)
{
  string	retStr;
  md5_byte_t	digest[16];
  md5_state_t	md5;
  
  md5_init(&md5);

  md5_append(&md5, (const md5_byte_t*) data.c_str(), data.length());

  md5_finish(&md5, digest);

  for (int c = 0; c < 16; c++) {
    retStr += charInt2hex(digest[c]);
  }

  return retStr;
}

Command
parseCommand(string str, string password, HostMap& hostInfo, unsigned int address)
{
  unsigned timeIndex = str.find(' ');
  if (timeIndex != string::npos) {
    unsigned hashIndex = str.find(' ', timeIndex + 1);

    if (hashIndex != string::npos) {
      if (digestMd5(str.substr(0, hashIndex) + str.substr(hashIndex, SALTLENGTH + 1 /* space in the beginning */) + password) ==
	  str.substr(hashIndex + 1 + SALTLENGTH)) {
	time_t sendTime = string2int(str.substr(timeIndex + 1, hashIndex - timeIndex - 1));

	if (hostInfo[address] == 0 ||
	    hostInfo[address] < sendTime) {
	  hostInfo[address] = sendTime;
	  if (str.substr(0, 4) == "LOCK") {
	    return LOCK;
	  } else if (str.substr(0, 6) == "UNLOCK") {
	    return UNLOCK;
	  }
	}
      } else {
	return INVALID;
      }
    } else {
      return INVALID;
    }
  } else {
    return INVALID;
  }
}

string
createCommand(Command command, string password)
{
  string answer;
  string salt = randomSalt();
  string timeStr = int2string(time(0));
  switch (command) {
  case LOCK: {
    answer = "LOCK";
  } break;
  case UNLOCK: {
    answer = "UNLOCK";
  } break;
  case INVALID: {
    // no such command, an error could be flagged..
    return answer;
  } break;
  }

  return answer + " " + timeStr + " " + salt + digestMd5(answer + " " + timeStr + " " + salt + password);
}
