#include "network.hh"
#include "lock.hh"

#include <iostream>

#include "config.hh"

using namespace std;

int
main(int argc, char** argv)
{
  if (argc < 3) {
    cout << "Usage: netlockd 'xlock &' 'killall xlock'\n";
    return 1;
  }
  
  HostMap hostInfo;
  
  string onLock = argv[1];
  string onUnlock = argv[2];
    
  try {
    int sock = createUdpSocket(PORT);
    bool locked = false;

    do {
      sockaddr_in address;
      
      string message = receiveString(sock, &address);

      Command command = parseCommand(message, PASSWORD, hostInfo, address.sin_addr.s_addr);

      switch (command) {
      case INVALID: {
	cout << "Invalid request!" << endl;
      } break;
      case LOCK: {
	if (!locked) {
	  locked = true;
	  system(onLock.c_str());
	}
      } break;
      case UNLOCK: {
	if (locked) {
	  locked = false;
	  system(onUnlock.c_str());
	}
      } break;
      }
    } while (true);
  } catch (NetworkException& e) {
    cout << "Network exception: " << e.what() << endl;
  }
  
  return 0;
}
