#include "network.hh"

#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

#include <unistd.h>

using namespace std;

int
createUdpSocket(int port)
{
  int sock = socket(PF_INET, SOCK_DGRAM, 17);

  int value = 1;
  if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &value, sizeof(value)) < 0) {
    throw NetworkException("Cannot set broadcast");
  }

  if (sock < 0) {
    throw NetworkException("No socket");
  }
  
  sockaddr_in address;
  address.sin_family = PF_INET;
  address.sin_addr.s_addr = INADDR_ANY;
  address.sin_port = htons(port);

  if (bind(sock, (sockaddr*)&address, sizeof(address)) < 0) {
    throw NetworkException("Cannot bind");
  }

  value = 1;
  if (setsockopt(sock, SOL_SOCKET, SO_BROADCAST, &value, sizeof(value)) < 0) {
    throw NetworkException("Cannot set broadcast");
  }
  
  return sock;
}

void
sendString(int sock, int port, std::string str, std::string host)
{
  sockaddr_in address;
  address.sin_family = PF_INET;

  if (host == "") {
    address.sin_addr.s_addr = INADDR_BROADCAST;
  } else {
    hostent* entry = gethostbyname(host.c_str());
    if (entry && entry->h_length == 4) {
      address.sin_addr.s_addr = htonl(((((unsigned int) (unsigned char) (entry->h_addr_list[0][0])) << 24) +
				       (((unsigned int) (unsigned char) (entry->h_addr_list[0][1])) << 16) +
				       (((unsigned int) (unsigned char) (entry->h_addr_list[0][2])) <<  8) +
				       (((unsigned int) (unsigned char) (entry->h_addr_list[0][3])) <<  0)));
    } else {
      throw NetworkException("Unknown host " + host);
    }
  }

  address.sin_port = htons(port);

  sendto(sock, str.c_str(), str.length(), 0, (sockaddr*)&address, sizeof(address));
  sendto(sock, str.c_str(), str.length(), 0, (sockaddr*)&address, sizeof(address));
  sendto(sock, str.c_str(), str.length(), 0, (sockaddr*)&address, sizeof(address));
}

string
receiveString(int sock, sockaddr_in* from)
{
  char buf[1024];
  int  bytes;

  if (!from) {
    bytes = recv(sock, buf, sizeof(buf) - 1, 0);
  } else {
    unsigned int fromLen = sizeof(sockaddr_in);
    bytes = recvfrom(sock, buf, sizeof(buf) - 1, 0, (sockaddr*) from, &fromLen);

    if (fromLen != sizeof(sockaddr_in)) {
      throw NetworkException("Missized network address");
    }
  }
  if (bytes >= 0) {
    buf[bytes] = 0;
    return string(buf);
  } else {
    return string("");
  }
}
